require("Game_Object")
require("Math_Util")

function new_enemy(game_area)
	local base = new_game_object(game_area)
	local base_render = base.render
	
	return
		game_object_extend(base,
		{
			-- Render material
			render_mat = new_material("material_enemy"),
		
			-- Enemy health
			health = 1,
			
			-- Mesh flashing
			mesh_flashing = 0,

			-- Updates the object
			update = function(this, delta_t)
				if not this:is_alive() then return false end
				
				-- Reduce the flashing
				this.mesh_flashing = lerp(this.mesh_flashing, 0, 0.2)
				
				return true
				end,
			
			-- Renders the object
			render = function(this, renderer)
				this.render_args["flashing"] = this.mesh_flashing
				base_render(this, renderer)
				end,
			
			-- Tests if the enemy is alive
			is_alive = function(this)
				return (this.health > 0)
				end,
			
			-- Applies damage to the enemy
			apply_damage = function(this, amt)
				if amt <= 0 or (not this:is_alive()) then return end
				
				-- Substract health
				this.health = this.health - amt
				
				-- Flash
				this.mesh_flashing = 1
				
				-- Sound effect
				resources:get("sound_enemy_hit"):play()
				
				-- Dead?
				if not this:is_alive() then
					this:on_death()
				end
				end,
				
			-- Called when the enemy is killed
			on_death = function(this, amt)
				end,
		})
end
