require("Enemy")
require("Math_Util")

function new_enemy_spinner(game_area, start_pos)
	local base = new_enemy(game_area)
	local base_update = base.update
	
	return
		game_object_extend(base,
		{
			-- Type of game object
			type = "enemy_spinner",
			
			-- Position for the game object
			pos = start_pos,
			
			-- Collision radius
			col_radius = 2,
			
			-- Render mesh (for default render behavior)
			render_mesh = "star",
			-- Render shader args
			render_args = { color = vec4(1,0.66,0.33,1) },
			
			-- Enemy health
			health = 5,

			rot_angle = 0,
			speed = rand_float(5, 10) * clamp(game_area.play_time * 0.1, 1, 10),
			
			-- Returns the current rendering matrix
			get_render_matrix = function(this)
				return 
					mat4_target_at(this.pos, vec3_add(this.pos, vec3(0,0,-1)), vec3(0,1,0)) *
					mat4_rot_y(this.rot_angle) *
					mat4_rot_x(0.08)
				end,
		
			-- Updates the object
			update = function(this, delta_t)
				if not base_update(this, delta_t) then return false end
				
				-- Rotate the enemy
				this.rot_angle = this.rot_angle + delta_t * 3
				
				-- Move the enemy
				this.pos = this.pos + delta_t * this.speed * vec3(0, 0, -1)
				
				return (this.pos[3] > -30)
				end,

			-- Called on collision
			on_collision = function(this, oth_obj)
				if oth_obj.type == "bullet" then
					this:apply_damage(1)
				end
				end,
				
			-- Called when the enemy is killed
			on_death = function(this, amt)
				game_area:add_score(100)
				resources:get("sound_enemy_death"):play()
				end,
			})
end
