require("Collider")
require("Game_Background")
require("Player")
require("Spawn_Manager")

function new_game_area()
	local function update_list(list, delta_t, collider)
		local new_list = {}
		for k,v in pairs(list) do
			if v:update(delta_t) then
				-- Keep object
				table.insert(new_list, v)
				collider:add_object(v)
			end
		end
		return new_list
	end

	local game_area =
		{
			collider = new_collider(),
			
			background = new_game_background(),
			
			bullets = {},
			enemies = {},
			
			score = 0,
			play_time = 0,
			difficulty = 0.5,
			
			size_x = 25,
			size_z = 15,
			spawn_z = 40,
			
			add_enemy = function(this, enemy)
				table.insert(this.enemies, enemy)
				end,
				
			add_bullet = function(this, bullet)
				table.insert(this.bullets, bullet)
				end,
			
			add_score = function(this, add_score)
				add_score = math.floor(add_score)
				if add_score > 0 then
					this.score = this.score + add_score
				end
				end,
			
			update = function(this, delta_t)
				-- Update the time played
				this.play_time = this.play_time + delta_t
				
				-- Update the background
				this.background:update(delta_t)
				
				-- Init the collider
				this.collider:reset()
				
				-- Update the player
				this.player:update(delta_t)
				if this.player.is_alive then
					this.collider:add_object(this.player)
				end
				
				-- Update the spawn manager
				this.spawn_manager:update(delta_t)
				
				-- Update the enemies
				this.enemies = update_list(this.enemies, delta_t, this.collider)
				
				-- Update the bullets
				this.bullets = update_list(this.bullets, delta_t, this.collider)
				
				-- Run the collisions
				this.collider:do_collisions()
				end,
				
			render = function(this, renderer)
				this.background:render(renderer)
				
				if this.player.is_alive then
					this.player:render(renderer)
				end
				
				for k,v in pairs(this.enemies) do
					v:render(renderer)
				end
				
				for k,v in pairs(this.bullets) do
					v:render(renderer)
				end
				end,
		}

	-- Add the player
	game_area.player = new_player(game_area)
	
	-- Add the spawn manager
	game_area.spawn_manager = new_spawn_manager(game_area)
	
	return game_area
end
