require("Material")
require("Table_Util")

function new_game_object(game_area)
	return
		{
			-- Type of game object
			type = "object",
			
			-- Position for the game object
			pos = vec3(),
			
			-- Collision mesh (if nil, then spherical collision is used). Meshes cannot collide with meshes.
			col_mesh = nil,
			-- Collision radius
			col_radius = 0,
			-- Collision enabled -flag
			col_enabled = true,
			
			-- Render mesh (for default render behavior)
			render_mesh = nil,
			-- Render material
			render_mat = new_material("material_normal"),
			-- Render shader args
			render_args = { color = vec4(1,1,1,1) },
			
			-- Returns the current rendering matrix
			get_render_matrix = function(this)
				return mat4_translation(this.pos)
				end,
				
			-- Returns the current collision matrix (by default same as rendering matrix)
			get_col_matrix = function(this)
				return this:get_render_matrix()
				end,
				
			-- Updates the object
			update = function(this, delta_t)
				return true
				end,
				
			-- Renders the object
			render = function(this, renderer)
				if this.render_mesh ~= nil then
					local obj_mtx = this:get_render_matrix()
					this.render_mat.shader_values = this.render_args
					renderer:add_mesh(this.render_mesh, obj_mtx, this.render_mat)
				end
				end,
				
			-- Called on collision
			on_collision = function(this, oth_obj)
				end,
		}
end

function game_object_extend(base_game_object, new_fields)
	return table_combine(base_game_object, new_fields)
end
