require("Math_Util")
require("Material")
require("Resources")

function new_screen_test(mesh_name)
	return
		{
			cam_angle = 0,
			cam_dist = 5,
			
			render_mat = new_material("material_normal"),
		
			update = function(this, delta_t)
				local mouse_x, mouse_y, mouse_b1, mouse_b2 = mouse_read()
				
				this.cam_angle = this.cam_angle + delta_t * 0.5
				
				-- Zoom in/out
				local zoom = 0
				if mouse_b1 then zoom = -1 elseif mouse_b2 then zoom = 1 end
				this.cam_dist = clamp(this.cam_dist + zoom * 10.0 * delta_t, 0.5, 20)
				
				return this
				end,
				
			render = function(this)
				local cam_pos = vec3(math.cos(this.cam_angle), 1, math.sin(this.cam_angle)):unit() * this.cam_dist
				local cam_mtx = mat4_target_at(cam_pos, vec3(0,0,0), vec3(0,1,0))
				local obj_mtx = mat4_look_at(vec3(0,0,0), vec3(0,0,100), vec3(0,1,0))
	
				render_bind_target()
				render_clear()
				render_set_camera(global_camera, cam_mtx)
				--render_set_shader(resources:get("shader_normal"))
				
				-- Render the mesh
				local mesh = resources:get(mesh_name)
				if mesh ~= nil then
					shader_set(this.render_mat.shader, "color", vec4(1, 1, 1, 1))
					render_mesh(mesh, obj_mtx, this.render_mat.appearance)
				end
				
				-- Render the axis lines
				render_set_shader(resources:get("shader_normal_line"))
				
				local bright = true
				for i = 0,100 do
					local c1 = i * 1
					local c2 = c1 + 1
					
					if bright then
						render_line(vec3(c1,0,0), vec3(c2,0,0), 0xff0000ff)
						render_line(vec3(0,c1,0), vec3(0,c2,0), 0x00ff00ff)
						render_line(vec3(0,0,c1), vec3(0,0,c2), 0x0000ffff)
					else
						render_line(vec3(c1,0,0), vec3(c2,0,0), 0x7f0000ff)
						render_line(vec3(0,c1,0), vec3(0,c2,0), 0x007f00ff)
						render_line(vec3(0,0,c1), vec3(0,0,c2), 0x00007fff)
					end
					bright = not bright
				end
				
				render_release_target()
				end,
		}
end
